@@ -422,7 +422,6 @@ ben\100morrow.me.uk                     mauzo\100csv.warwick.ac.uk
 +                                       mauzo\100.(none)
 bepi\100perl.it                         enrico.sorcinelli\100gmail.com
 bert\100alum.mit.edu                    bert\100genscan.com
-bigbang7\100gmail.com                   ddascalescu+github\100gmail.com
 brian.d.foy\100gmail.com                bdfoy\100cpan.org
 BQW10602\100nifty.com                   sadahiro\100cpan.org
 
@@ -550,6 +549,7 @@ laszlo.molnar\100eth.ericsson.se        molnarl\100cdata.tvnet.hu
 +                                       ml1050\100freemail.hu
 lewart\100uiuc.edu                      lewart\100vadds.cvm.uiuc.edu    
 +                                       d-lewart\100uiuc.edu
+lkundrak\100v3.sk                      lubo.rintel\100gooddata.com
 lstein\100cshl.org                      lstein\100formaggio.cshl.org
 +                                       lstein\100genome.wi.mit.edu
 lupe\100lupe-christoph.de               lupe\100alanya.m.isar.de
@@ -574,7 +574,6 @@ mikedlr\100tardis.ed.ac.uk              mikedlr\100it.com.pl
 moritz\100casella.verplant.org          moritz\100faui2k3.org
 +                                       moritz lenz
 
-mst\100shadowcat.co.uk                  matthewt\100hercule.scsys.co.uk
 neale\100VMA.TABNSW.COM.AU              neale\100pucc.princeton.edu
 neeracher\100mac.com                    neeri\100iis.ee.ethz.ch
 neil\100bowers.com                      neilb\100cre.canon.co.uk
@@ -654,6 +653,7 @@ rmgiroux\100acm.org                     rmgiroux\100hotmail.com
 rmbarker\100cpan.org                    rmb1\100cise.npl.co.uk
 +                                       robin.barker\100npl.co.uk
 +                                       rmb\100cise.npl.co.uk
++                                       robin\100spade-ubuntu.(none)
 robertmay\100cpan.org                   rob\100themayfamily.me.uk
 roberto\100keltia.freenix.fr            roberto\100eurocontrol.fr
 robin\100cpan.org                       robin\100kitsite.com
