@@ -11,10 +11,6 @@
 ;#	or
 ;#	@words = shellwords();		# defaults to $_ (and clobbers it)
 
-warn( "The 'shellwords.pl' legacy library is deprecated and will be"
-      . " removed in the next major release of perl. Please use the"
-      . " Text::ParseWords module instead." );
-
 require Text::ParseWords;
 *shellwords = \&Text::ParseWords::old_shellwords;
 
