@@ -7,22 +7,19 @@
 ;#	$st_dev = @ary[$ST_DEV];
 ;#
 
-warn( "The 'stat.pl' legacy library is deprecated and will be"
-      . " removed in the next major release of perl." );
-
-$ST_DEV =	0 + $[;
-$ST_INO =	1 + $[;
-$ST_MODE =	2 + $[;
-$ST_NLINK =	3 + $[;
-$ST_UID =	4 + $[;
-$ST_GID =	5 + $[;
-$ST_RDEV =	6 + $[;
-$ST_SIZE =	7 + $[;
-$ST_ATIME =	8 + $[;
-$ST_MTIME =	9 + $[;
-$ST_CTIME =	10 + $[;
-$ST_BLKSIZE =	11 + $[;
-$ST_BLOCKS =	12 + $[;
+$ST_DEV =	0;
+$ST_INO =	1;
+$ST_MODE =	2;
+$ST_NLINK =	3;
+$ST_UID =	4;
+$ST_GID =	5;
+$ST_RDEV =	6;
+$ST_SIZE =	7;
+$ST_ATIME =	8;
+$ST_MTIME =	9;
+$ST_CTIME =	10;
+$ST_BLKSIZE =	11;
+$ST_BLOCKS =	12;
 
 ;# Usage:
 ;#	require 'stat.pl';
