@@ -155,8 +155,8 @@ sub Tgoto {
 	}
 	elsif ($code eq '>') {
 	    ($code,$tmp,$string) = unpack("CCa99",$string);
-	    if ($tmp[$[] > $code) {
-		$tmp[$[] += $tmp;
+	    if ($tmp[0] > $code) {
+		$tmp[0] += $tmp;
 	    }
 	}
 	elsif ($code eq '2') {
